@@ -30,7 +30,7 @@
 #include <fribidi-unicode.h>
 
 #include <stdio.h>
-#if STDC_HEADERS+0
+#ifdef STDC_HEADERS
 # include <stdlib.h>
 # include <stddef.h>
 #else
@@ -38,13 +38,13 @@
 #  include <stdlib.h>
 # endif
 #endif
-#if HAVE_STRING_H+0
+#ifdef HAVE_STRING_H
 # if !STDC_HEADERS && HAVE_MEMORY_H
 #  include <memory.h>
 # endif
 # include <string.h>
 #endif
-#if HAVE_STRINGS_H+0
+#ifdef HAVE_STRINGS_H
 # include <strings.h>
 #endif
 
@@ -208,7 +208,6 @@ read_data (
   for (; data_file_name[0] && data_file_type[0];
        data_file_name++, data_file_type++)
     {
-      fprintf (stderr, "Reading `%s'\n", data_file_name[0]);
       if (!(f = fopen (data_file_name[0], "rt")))
 	die2 ("error: cannot open `%s' for reading", data_file_name[0]);
 
@@ -234,8 +233,6 @@ gen_arabic_shaping_tab (
   if (maxshaped < minshaped)
     die ("error: no shaping pair found, something wrong with reading input");
 
-  fprintf (stderr,
-	   "Generating `" outputname "', it may take up to a few minutes\n");
   printf ("/* " outputname "\n * generated by " appname " (" FRIBIDI_NAME " "
 	  FRIBIDI_VERSION ")\n" " * from the files %s, %s of Unicode version "
 	  FRIBIDI_UNICODE_VERSION ". */\n\n", data_file_type[0],
